from geocelery import Function

def xxx():
    import time
    while True:
        print("hello,world")
        time.sleep(1)

if __name__ == "__main__":


    #fbb31332-c487-4918-8652-b5555c51f6c1
    #f2237140-548d-4dbf-9e85-9b8e79bb83a5
    #5e070209-0dd7-4b4d-8e67-8ebce6a45ac5
    Function.stop("ae951335-4b20-49c2-b6ba-e0a58030251a")

    # taskid = Function.start(xxx)
    #
    # import time
    # time.sleep(5)
    #
    # print(Function.status(taskid))
    #
    # Function.stop(taskid)
    #
    # print(Function.status(taskid))
